/**
 * LOGICAL-PARADOX.ORG variable record manager library
 * Copyright (C)2004 satoshi akabane(akabane@logical-paradox.org)
 *
 * Release 1.0
 * Ep[W
 */
package org.logical_paradox.common.vrm;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.logical_paradox.common.io.CommonRandomAccessFileDelegator;
import org.logical_paradox.common.io.RandomAccessFileDelegator;
import org.logical_paradox.common.thread.ReadWriteLock;

/**
 * ϒR[ht@C}l[W[
 * ϒTCỸR[ht@C̓o͊ǗsNX
 * 
 * @author satoshi akabane@logical-paradox.org
 * @version 1.0
 */
public class VariableRecordManager {
	protected String filename;
	protected RandomAccessFileDelegator file;
	protected VRMAvailableList availableList;
	protected RecordFactory factory;

	private ReadWriteLock lock = new ReadWriteLock();

	/**
	 * RXgN^
	 * @param filename ǗΏۂ̃t@C
	 * @throws IOException t@CANZXɎs
	 */
	public VariableRecordManager(String filename, RecordFactory factory) throws IOException {
		this.filename = filename;
		this.factory = factory;
		
		open();
	}
	/**
	 * t@CI[v
	 * hNXŕʂ̃^Cṽt@CI[vꍇC̃\bhI[o[Ch
	 * @throws IOException I[vɎs
	 */
	public void open() throws IOException {
		// ǂݏ[hRandomAccessFileI[v
		file = new CommonRandomAccessFileDelegator(filename, "rw");
		availableList = new VRMAvailableList(filename + ".vra");
	}
	/**
	 * t@C
	 * @throws IOException t@CANZXɎs
	 */
	public void close() throws IOException {
		// f[^t@C
		if(file != null) {
			file.close();
			file = null;
		}
		// p\Xgo͂
		if(availableList != null) {
			availableList.flush();
			availableList = null;
		}
	}
	/**
	 * t@CɐVR[hǉ
	 * 
	 * @param r ǉR[h
	 * @return Vɍ쐬ꂽR[h̃oCgψ
	 * @throws IOException R[h̒ǉɎs
	 */
	public long insert(Record r) throws IOException {
		long offset = -1;
		try {
			lock.beginWrite();

			// ėp\ȃR[h邩ǂ𒲂ׂ
//			AvailableRecord emptyRecord = (AvailableRecord)availableList.getCache().getOne();
//			if(emptyRecord == null) {
	AvailableRecord emptyRecord = null;
				offset = file.length();
//			} else if(emptyRecord.getRecordSize() < r.size()) {
//				// 擾p\R[h̃TCY\łȂꍇ
//				// xoĂ܂̂ŁCēxǉĂ
//				offset = file.length();
//				availableList.getCache().add(emptyRecord);
//			} else {
//				offset = availableList.getOffset();
//			}
			// V[N
			synchronized(file) {
				file.seek(offset);
				// t@C̏I[ɃV[Nďo͂
				file.write(r.byteStream());
			}
			// t@C̓rɏ㏑ăR[h쐬ꍇCVp\R[h݂̐
			if(emptyRecord != null) {
				int emptyRecordSize = emptyRecord.getRecordSize() - r.getRecordSize();
				if(emptyRecordSize > 0) {
					availableList.getCache().add(new AvailableRecord(emptyRecordSize, offset+r.size()));
				}
			}
		} catch(InterruptedException e) {
		} finally {
			try {
				lock.release();
			} catch (InterruptedException e1) {
			}
		}
		return offset;
	}
	/**
	 * w肳ꂽR[hXV
	 * ȑOƍxœR[hTCY̏ꍇCSɍXVD<br>
	 * ȊȌꍇCȑÕR[h폜C폜ꂽ̈͒fЉD
	 * @param rno R[hԍ
	 * @param r R[h
	 * @throws IOException XVɎs
	 */
	public void update(long rno, Record r) throws IOException {
		Record pre = get(rno);
		if(pre.size() == r.size()) {
			// TCYS̏ꍇ
		} else {
			delete(rno);
			insert(r);
		}
	}
	/**
	 * w肳ꂽR[h擾
	 * @param rno R[hԍ
	 * @return R[h
	 * @throws IOException R[h̎擾Ɏs
	 */
	public Record get(long rno) throws IOException {
		try {
			lock.beginRead();
			// Ƃ肠YʒuփV[N
			synchronized(file) {
				file.seek(rno);
				// R[hTCYi[ĂC擪4oCg܂擾
				byte[] recordSizeBuffer = new byte[4];
				file.read(recordSizeBuffer);
		
				// R[hTCY𕜌邽߂ɁCByteBuffer𐶐
				ByteBuffer buffer = ByteBuffer.allocate(4).order(ByteOrder.BIG_ENDIAN);
				buffer.rewind();
				buffer.put(recordSizeBuffer);
				buffer.rewind();

				int recordSize = buffer.getInt();
				byte[] recordBody = new byte[recordSize];

				// ݃|CgĂV[N|C^CKvȃf[^擾
				file.read(recordBody);
				// t@Ng𗘗păoCg̃R[hIuWFNgɃCX^X
				return factory.getRecord(recordBody);
			}
		} catch(InterruptedException e) {
		} finally {
			try {
				lock.release();
			} catch(InterruptedException e) {
			}
		}
		return null;
	}
	/**
	 * w肳ꂽR[h폜
	 * @param rno 폜R[hԍ
	 * @throws IOException 폜Ɏs
	 */
	public void delete(long rno) throws IOException {
		// ł͒PɎw肳ꂽR[h𗘗p\Xgɒǉ邾
		// ȏ͕̏KvȂ
		Record r = get(rno);
		AvailableRecord ar = new AvailableRecord(r.getRecordSize(), rno);
		availableList.getCache().add(ar);
	}
}
