/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: FormatCommand.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.dac.command.handler;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.common.session.IllegalSessionStateException;
import org.logical_paradox.common.session.Session;
import org.logical_paradox.vsan.dac.DacConstant;
import org.logical_paradox.vsan.dac.NodeContext;
import org.logical_paradox.vsan.dac.Response;
import org.logical_paradox.vsan.dac.ResponseCode;
import org.logical_paradox.vsan.dac.command.exception.CommandExecutionException;
import org.logical_paradox.vsan.dac.command.exception.SessionTerminationException;
import org.logical_paradox.vsan.node.Database;

/**
 * tH[}bgR}h
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class FormatCommand implements DacCommand {
	/** K[ */
	private static final Log log = LogFactory.getLog(FormatCommand.class);

	/**
	 * R}hs
	 * @param session sZbV
	 * @return XVꂽZbV
	 * @throws CommandExecutionException R}hsɔO
	 * @throws SessionTerminationException ڑIɔO
	 */
	public Session execute(Session session) throws CommandExecutionException, SessionTerminationException {
		try {
			NodeContext ctx = (NodeContext)session.getAttribute(DacConstant.SKEY_NODE_CONTEXT);
			Database db = ctx.getDatabase();

			// ̎擾
			byte[] data = new byte[32 + 4];
			InputStream in = (InputStream)session.getAttribute(DacConstant.SKEY_SOCKET_INPUT_STREAM);
			int received = in.read(data);

			if(received != data.length) {
				log.warn("{[" + data.length + "]oCgMĂf[^A[" + received + "]oCgMł܂ł");
				throw new CommandExecutionException();
			}

			ByteBuffer b = ByteBuffer.allocate(data.length).order(ByteOrder.BIG_ENDIAN);
			b.put(data);
			b.rewind();

			// hC
			byte[] domain = new byte[32];
			b.get(domain);
			String domainname = new String(domain).trim();

			if(domainname.indexOf(".") >= 0) {
				throw new CommandExecutionException("illegal domain name");
			}

			// R[hTCY
			int recordSize = b.getInt();
			if(recordSize <= 0) {
				throw new CommandExecutionException("illegal record size");
			}
			// tH[}bg
			db.format(domainname, DacConstant.MAJOR_VERSION, DacConstant.MINOR_VERSION, recordSize);

			// X|XR[h̐ݒ
			session.setAttribute(DacConstant.SKEY_DAC_RESPONSE, new Response(ResponseCode.OK, new byte[0]));
		} catch (IllegalSessionStateException e) {
			// ZbVp\ȏԂłȂꍇCYZbVI
			log.warn("hCf[^x[X̏v܂AZbV\ȏԂł͂܂łB");
			log.warn("ZbVj܂B");
			throw new SessionTerminationException(e);
		} catch (IOException e) {
			log.warn("R}hMI/OO܂");
			throw new CommandExecutionException(e);
		}

		return session;
	}

}
