/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: Database.java,v 1.4 2005/10/11 16:13:34 rampil Exp $
 */
package org.logical_paradox.vsan.node;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.common.io.CommonRandomAccessFileDelegator;
import org.logical_paradox.common.io.RandomAccessFileDelegator;
import org.logical_paradox.common.util.StringUtils;
import org.logical_paradox.vsan.dac.DacConfig;
import org.logical_paradox.vsan.dac.DacConstant;
import org.logical_paradox.vsan.dac.DacProperty;
import org.logical_paradox.vsan.dac.DomainSynchronizer;
import org.logical_paradox.vsan.dac.NodeContext;
import org.logical_paradox.vsan.dac.ResponseCode;
import org.logical_paradox.vsan.node.api.DataStreamIoApi;
import org.logical_paradox.vsan.node.dbr.DbFile;
import org.logical_paradox.vsan.node.dbr.DbRecord;
import org.logical_paradox.vsan.node.dbr.RecordSet;

/**
 * hCf[^x[Xt@C
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.4 $
 */
public class Database {
	/** K[ */
	private static final Log log = LogFactory.getLog(Database.class);

	/** }WbNio[ */
	public static final String MAGIC_NUMBER = "VSDD";

	/** QĂhC */
	private String domain;
	/** t@CtH[}bg̃W[o[W */
	private int majorVersion;
	/** t@CtH[}bg̃}Ci[o[W */
	private int minorVersion;
	/** R[h̃TCY */
	private int recordSize = 0;
	/** ŏIœK */
	private Timestamp lastOptimizeTime;
	/** f[^XgAĂȂR[hԍQ̊Jnʒu(=ݗp̍ő̃R[hԍ+1) */
	private long recordBeginOfEmpties = 1;

	/** f[^t@C */
	private List datafiles = new ArrayList();
	/** m[hReLXg */
	private final NodeContext context;
	/** _ANZXt@C */
	private RandomAccessFileDelegator file;


	/*==========================================================================
	 * API
	 *========================================================================**/
	/** f[^oAPI */
	private DataStreamIoApi apiDataStreamIo;

	/**
	 * RXgN^
	 * @param ctx fBXNACm[hReLXg
	 */
	public Database(NodeContext ctx) throws DatabaseAccessException {
		context = ctx;
		createApi();
	}
	/**
	 * f[^x[XpAPIQ𐶐
	 */
	private void createApi() throws DatabaseAccessException {
		log.info("API[h܂");
		try {
			// f[^Xg[I/O API[h
			Class clazz = Class.forName(DacProperty.getProperty(DacConstant.PKEY_API_DATASTREAM_IO));
			Constructor constructor = clazz.getConstructor(new Class[]{Database.class});
			apiDataStreamIo = (DataStreamIoApi)constructor.newInstance(new Object[]{this});
			log.info("f[^Xg[I/O API[h܂");
		} catch (Exception e) {
			log.error("f[^Xg[I/O APĨ[hɎs܂");
			throw new DatabaseAccessException(ResponseCode.DATABASE_SYSTEM_ERROR);
		}
		log.info("APĨ[h܂");
	}
	/*=========================================================================
	 * DiskArrayController֒񋟂API
	 *========================================================================*/
	/**
	 * t@CN[Y
	 */
	public void close() {
		if(file != null) {
			log.info("f[^x[XĂ܂");
			synchronized(file) {
				// SẴf[^t@CSɕ
				for(Iterator it = datafiles.iterator(); it.hasNext();) {
					DbFile dbf = (DbFile)it.next();
					try {
						dbf.close();
					} catch (IOException e1) {
						log.warn("f[^t@C[" + dbf.getFilename() + "]̃N[YI/OO܂", e1);
					}
				}
				try {
					file.close();
				} catch (IOException e) {
					// 
				} finally {
					file = null;
					log.info("f[^x[X܂");
				}
			}
			try {
				store();
			} catch (IOException e) {
				log.warn("f[^x[Xt@C̃N[YɎs܂", e);
			}
		}
	}
	/**
	 * f[^x[XtH[}bg
	 * f[^x[Xt@C͍̏sȂ(wb_̂)
	 * @param dn hC
	 * @param major W[o[W
	 * @param minor }Ci[o[W
	 * @param rs R[hTCY
	 */
	public void format(String dn, int major, int minor, int rs) throws IOException {
		log.info("hCf[^x[Xt@CtH[}bgĂ܂");
		setDomain(dn);
		setMajorVersion(major);
		setMinorVersion(minor);
		setRecordSize(rs);
		recordBeginOfEmpties = 1;

		// wb_o͂
		store();
		log.info("hCf[^x[Xt@C̃tH[}bg܂");

		// hCɎQĂꍇChC̗EʒB
		if(context.isJoined() == true) {
			DomainSynchronizer.leave(context);
		}
	}
	/**
	 * hCf[^x[Xj
	 * @throws IOException
	 */
	public void dispose() throws IOException {
		log.info("hCf[^x[XjĂ܂");

		// hCɎQĂꍇCɃhC̗EʒB
		if(context.isJoined() == true) {
			DomainSynchronizer.leave(context);
		}

		synchronized(file) {
			// ŏɑSẴf[^t@CƃhCf[^x[Xt@CN[Y
			close();

			for(Iterator it = datafiles.iterator(); it.hasNext();) {
				DbFile dbf = (DbFile)it.next();
				if(new File(context.getConfig().getDatabaseDir() + File.separator + dbf.getFilename()).delete() == false) {
					throw new IOException("[" + dbf.getFilename() + "]̍폜Ɏs܂B蓮ō폜ĂB");
				}
			}

			// hCf[^x[Xt@C̍폜
			if(new File(context.getConfig().getDatabaseDir() + File.separator + getFilename()).delete() == false) {
				throw new IOException("[" + getFilename() + "]̍폜Ɏs܂B蓮ō폜ĂB");
			}

			// f[^t@CɊւIuWFNgSď
			file = null;
			domain = null;
			majorVersion = minorVersion = 0;
			lastOptimizeTime = null;
			recordSize = 0;
			recordBeginOfEmpties = 1;
			datafiles.clear();
		}

		log.info("hCf[^x[Xj܂");
	}
	/**
	 * Vf[^x[Xt@C̃m[hɒǉ
	 * @param recordBegin JnR[hԍ
	 * @param recordEnd IR[hԍ
	 */
	public void expand(long recordBegin, long recordEnd) throws IOException {
		synchronized(file) {
			log.info("f[^t@CǉĂ܂");
			DbFile dbf = new DbFile(recordBegin, recordEnd, recordSize);
			// f[^x[Xt@C
			dbf.init(context.getConfig().getDatabaseDir());
			datafiles.add(dbf);

			// wb_o͂
			store();
			log.info("f[^t@C̒ǉ܂[" + dbf.getFilename() + "]");
		}

		// hCɎQĂꍇChCɍXVeʒm
		if(context.isJoined() == true) {
			DomainSynchronizer.sync(context);
		}
	}
	/**
	 * f[^x[Xt@C̍œK{
	 * fЉꂽR[hAIȈʒuֈړ
	 * @throws IOException œKI/OG[
	 */
	public void optimize() throws IOException {
		synchronized(file) {
			// ŏIœKXV
			setLastOptimizeTime(new Timestamp(System.currentTimeMillis()));

			// wb_o͂
			store();
		}
	}
	/**
	 * w肳ꂽR[hԍ擪ɁA̓Xg[f[^擾ăf[^t@C֏
	 * @param recordBegin JnR[hԍ
	 * @param in f[^\[X̓Xg[
	 * @param size f[^TCY
	 * @throws DatabaseAccessException p[^̕sȂǂŏ𑱍słȂ
	 * @throws IOException NCAgƂ̒ʐMɏQ
	 */
	public void storeStream(long recordBegin, InputStream in, int size) throws DatabaseAccessException, IOException {
		long recordEnd = apiDataStreamIo.storeStream(recordBegin, in, size);
		recordBeginOfEmpties = recordEnd+1;
	}
	/**
	 * w肳ꂽԍ̃R[hԂ
	 * @param recordno R[hԍ̔z
	 * @return ꂽR[h̔z
	 * @throws DatabaseAccessException p[^̕sȂǂŏsłȂ
	 * @throws IOException NCAgƂ̒ʐMɏQ
	 */
	public DbRecord[] restoreStream(long[] recordno) throws DatabaseAccessException, IOException {
		return apiDataStreamIo.restoreStream(recordno);
	}
	/**
	 * fBXNACɊւԂD
	 * @return fBXNACɊւi[IuWFNgD
	 */
	public DiskArrayNode getNodeInfo() {
		DiskArrayNode node = new DiskArrayNode();
		DacConfig config = context.getConfig();

		// m[h
		node.setHost(config.getHost() + ":" + config.getPort());
		// R[hZbg
		synchronized(datafiles) {
			for(Iterator it = datafiles.iterator(); it.hasNext();) {
				node.add((RecordSet)it.next());
			}
		}

		return node;
	}
	/*=========================================================================
	 * DatabaseNXŎgp鉺ʃT[rX[`
	 *========================================================================*/
	/**
	 * f[^x[Xt@C𕨗t@Cɏo͂
	 * @throws IOException o̓G[
	 */
	protected void store() throws IOException {
		log.info("hCf[^x[Xt@CXVĂ܂");

		ByteBuffer buffer = ByteBuffer.allocate(4096).order(ByteOrder.BIG_ENDIAN);

		// }WbNio[
		buffer.put(MAGIC_NUMBER.getBytes());
		// tH[}bgo[W(W[o[W)
		buffer.put((byte)majorVersion);
		// tH[}bgo[W(}Ci[o[W)
		buffer.put((byte)minorVersion);
		// R[hTCY
		buffer.putInt(recordSize);
		// gp̈̐擪
		buffer.putLong(recordBeginOfEmpties);
		// ŏIœK
		if(lastOptimizeTime == null) {
			lastOptimizeTime = new Timestamp(System.currentTimeMillis());
		}
		buffer.putLong(lastOptimizeTime.getTime());
		// hC
		buffer.put((domain + "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0").substring(0, 32).getBytes());

		// f[^t@C
		buffer.put((byte)datafiles.size());
		for(Iterator it = datafiles.iterator(); it.hasNext();) {
			DbFile dbf = (DbFile)it.next();
			// JnR[h
			buffer.putLong(dbf.getRecordBegin());
			// IR[h
			buffer.putLong(dbf.getRecordEnd());
		}

		if(file == null) {
			file = createPhysicalFileDelegator(context.getConfig().getDatabaseDir() + File.separator + getFilename());
		} else {
			file.seek(0);
		}
		file.write(buffer.array());

		log.info("hCf[^x[Xt@C̍XV܂");
	}
	/**
	 * f[^x[Xt@C̓e荞
	 * @throws IOException o̓G[
	 */
	public void restore() throws IOException {
		if(file != null) {
			file.seek(0);
		} else {
			file = createPhysicalFileDelegator(context.getConfig().getDatabaseDir() + File.separator + getFilename());
		}
		if(file.length() == 0L) {
			return;
		}

		log.info("hCf[^x[Xt@Cǂݍł܂");

		byte[] buf = new byte[4096];
		file.read(buf);

		ByteBuffer buffer = ByteBuffer.allocate(buf.length).order(ByteOrder.BIG_ENDIAN);
		buffer.put(buf);
		buffer.rewind();

		// }WbNio[
		String mh = new String(new byte[]{ buffer.get(), buffer.get(), buffer.get(), buffer.get() });
		if(MAGIC_NUMBER.equals(mh) == false) {
			throw new IOException();
		}
		// tH[}bgo[W(W[)
		setMajorVersion((int)buffer.get());
		// tH[}bgo[W(}Ci[)
		setMinorVersion((int)buffer.get());
		// R[hTCY
		setRecordSize(buffer.getInt());
		// gp̈̐擪
		recordBeginOfEmpties = buffer.getLong();
		// ŏIœK
		setLastOptimizeTime(new Timestamp(buffer.getLong()));
		// hC
		byte[] dn = new byte[32];
		buffer.get(dn);
		setDomain(new String(dn).trim());

		// f[^t@C
		int filecnt = buffer.get();
		for(int i = 0; i < filecnt; i++) {
			long recordBegin = buffer.getLong();
			long recordEnd = buffer.getLong();

			// f[^t@CJԂɂ
			DbFile dbf = new DbFile(recordBegin, recordEnd, recordSize);
			dbf.open(context.getConfig().getDatabaseDir());
			datafiles.add(dbf);
		}

		log.info("hCf[^x[Xt@C̓ǂݍ݂܂");
	}
	/**
	 * f[^t@C̃XgԂ
	 * pbP[Wɑ݂ĂNX̂݃ANZX\
	 * @return f[^t@C̃Xg
	 */
	List getDatafiles() {
		return datafiles;
	}
	/**
	 * f[^t@Cǉ
	 * @param dbf f[^t@C
	 */
	protected void addDataFile(DbFile dbf) {
		datafiles.add(dbf);
	}
	/**
	 * f[^x[Xt@C̖OԂ
	 * @return t@C
	 */
	public String getFilename() {
		return context.getNodeId() + "." + getDomain() + DacConstant.DBI_EXT;
	}
	/**
	 * f[^t@C̃t@CԂ
	 * @return f[^t@C̏
	 */
	public String[] getDatafileSpec() {
		synchronized(datafiles) {
			String[] dbfspecs = new String[datafiles.size()];
			int i = 0;
			for(Iterator it = datafiles.iterator(); it.hasNext(); i++) {
				DbFile dbf = (DbFile)it.next();
				dbfspecs[i] = dbf.toString();
			}
			return dbfspecs;
		}

	}
	/**
	 * hCԂ
	 * @return hC
	 */
	public String getDomain() {
		return domain;
	}

	/**
	 * ŏIœKԂ
	 * @return ŏIœK
	 */
	public Timestamp getLastOptimizeTime() {
		return lastOptimizeTime;
	}

	/**
	 * t@CtH[}bg̃W[o[WԍԂ
	 * @return W[o[W
	 */
	public int getMajorVersion() {
		return majorVersion;
	}

	/**
	 * t@CtH[}bg̃}Ci[o[WԍԂ
	 * @return }Ci[o[W
	 */
	public int getMinorVersion() {
		return minorVersion;
	}

	/**
	 * R[hTCYԂ
	 * @return R[hTCY
	 */
	public int getRecordSize() {
		return recordSize;
	}

	/**
	 * hCݒ肷
	 * 1xݒłȂ
	 * @param string hC
	 */
	public void setDomain(String string) {
		if(StringUtils.isEmpty(domain) == true) {
			domain = string;
		}
	}

	/**
	 * ŏIœKݒ肷
	 * @param timestamp ŏIœK
	 */
	private void setLastOptimizeTime(Timestamp timestamp) {
		lastOptimizeTime = timestamp;
	}

	/**
	 * t@CtH[}bg̃W[o[Wݒ肷
	 * 1xݒłȂ
	 * @param i W[o[W
	 */
	public void setMajorVersion(int i) {
		if(majorVersion <= 0) {
			majorVersion = i;
		}
	}

	/**
	 * t@CtH[}bg̃}Ci[o[Wݒ肷
	 * 1xݒłȂ
	 * @param i }Ci[o[W
	 */
	public void setMinorVersion(int i) {
		if(minorVersion <= 0) {
			minorVersion = i;
		}
	}

	/**
	 * R[hTCYݒ肷
	 * Cɐݒ肳Ăꍇ͕ύXłȂ
	 * @param i R[hTCY
	 */
	public void setRecordSize(int i) {
		if(recordSize <= 0) {
			recordSize = i;
		}
	}

	/**
	 * gpR[hԍ̐擪ʒuԂD
	 * ݃f[^XgAĂ郌R[hԍ̏I[ʒu+1̃R[hԍԋpD
	 * @return gpR[hԍ̐擪ʒu
	 */
	public long getRecordBeginOfEmpties() {
		return recordBeginOfEmpties;
	}

	/**
	 * t@Cɑ΂fQ[^[𐶐ĕԂ
	 * @param filename t@C
	 * @return ꂽfQ[^[
	 * @throws FileNotFoundException t@C݂Ȃ
	 */
	protected RandomAccessFileDelegator createPhysicalFileDelegator(String filename) throws FileNotFoundException {
		return new CommonRandomAccessFileDelegator(filename, "rw");
	}
}
