/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DbFile.java,v 1.3 2005/09/17 17:35:49 rampil Exp $
 */
package org.logical_paradox.vsan.node.dbr;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.common.io.CommonRandomAccessFileDelegator;
import org.logical_paradox.common.io.FileUtils;
import org.logical_paradox.common.io.RandomAccessFileDelegator;
import org.logical_paradox.common.util.ByteStreamConverter;
import org.logical_paradox.vsan.dac.DacConstant;

/**
 * ̃R[h͈͂f[^t@C𒊏ۉĎ舵IuWFNg
 * f[^t@Cւ̓o͂́C̃NXĊԐړIɎ{D
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.3 $
 */
public class DbFile extends RecordSet implements Comparable {
	/** K[ */
	private static final transient Log log = LogFactory.getLog(DbFile.class);

	/** _ANZXt@CfQ[^[ */
	private transient RandomAccessFileDelegator file;
	/** ݂̃V[N|C^|CgĂ郌R[hԍ */
	private transient long currentRecordNo = 0;

	/**
	 * RXgN^
	 * @param recordBegin_ R[h͈(Jn)
	 * @param recordEnd_ R[h͈(I)
	 */
	public DbFile(long recordBegin_, long recordEnd_, int recordSize_) {
		super(recordBegin_, recordEnd_, recordSize_);
		currentRecordNo = recordBegin_;
	}
	/**
	 * f[^t@CJ
	 * @param dir f[^t@C̔zuʒu
	 * @throws IOException f[^t@CJȂ
	 */
	public void open(String dir) throws IOException {
		// ̂̂΁CÔ߈xN[YĂ
		close();
		file = createPhysicalFileDelegator(dir + File.separator + getFilename());
		log.trace("t@CI[v܂");
	}
	/**
	 * f[^t@C
	 * @throws IOException f[^t@C̃N[YɎs
	 */
	public void close() throws IOException {
		if(file != null) {
			file.close();
			log.trace("t@CN[Y܂");
		}
	}
	/**
	 * f[^t@C
	 * f[^t@C͕KꂽԂŌĂяoȂ΂ȂȂ
	 * @param dir fBNg
	 * @throws IOException s
	 */
	public void init(String dir) throws IOException {
		log.info("f[^t@C쐬Ă܂");
		FileUtils.fill(dir + File.separator + getFilename(), (int)(getRecordEnd() - getRecordBegin()), 0);
		log.info("f[^t@C̍쐬܂");
	}
	/**
	 * t@Cɑ΂fQ[^[𐶐ĕԂ
	 * TuNX́C̃\bhI[o[Ch邱Ƃŉzt@Cւ̃fQ[^[<br>
	 * ԂƂł
	 * @param filename t@C
	 * @return ꂽfQ[^[
	 * @throws FileNotFoundException w肳ꂽt@C݂Ȃ
	 */
	protected RandomAccessFileDelegator createPhysicalFileDelegator(String filename) throws FileNotFoundException {
		return new CommonRandomAccessFileDelegator(filename, "rw");
	}
	/**
	 * ̃f[^t@C̕\Ԃ
	 * @return \
	 */
	public String toString() {
		return getFilename();
	}
	/**
	 * t@CԂ
	 * @return t@C
	 */
	public String getFilename() {
		return super.toString() + DacConstant.DBF_EXT;
	}
	/**
	 * f[^t@Cɑ΂āAw肳ꂽR[h
	 * Ȃׂނɂ́AR[hԍŃ\[gĂKv
	 * @param records R[h̃Xg(gDbRecordł邱)
	 */
	public void store(List records) throws IOException {
		synchronized(file) {
			// w肳ꂽR[h́C{Iɂ͘Ä悾Ƃ̑O
			// ݃bN擾Sď
			for(Iterator it = records.iterator(); it.hasNext();) {
				DbRecord record = (DbRecord)it.next();

				long recordNo = record.getRecordNo();
				if(recordNo != currentRecordNo) {
					// V[N|C^ړ(ÄłȂꍇ̂)
					long seekp = (recordNo - getRecordBegin()) * (getRecordSize() +4);		// +4 = f[^TCY(int)
					file.seek(seekp);
				}
				// f[^̏
				file.write(ByteStreamConverter.convert(record.getDatasize()));
				file.write(record.getData());
				currentRecordNo = recordNo +1;
			}
		}
		
	}
	/**
	 * w背R[h̃f[^𕜌
	 * @param recordNo R[hԍ
	 * @return ꂽR[hԍ
	 */
	public DbRecord restore(long recordNo) throws IOException {
		synchronized(file) {
			if(currentRecordNo != recordNo) {
				// V[N|C^ړ(ÄłȂꍇ̂)
				long seekp = (recordNo - getRecordBegin()) * (getRecordSize() +4);
				file.seek(seekp);
			}
			// f[^̕
			byte[] datasize = new byte[4];
			file.read(datasize);

			byte[] bstream = new byte[getRecordSize()];
			file.read(bstream);

			DbRecord record = new DbRecord(recordNo, getRecordSize(), bstream, ByteStreamConverter.getInt(datasize));
			return record;
		}
	}

	/**
	 * f[^x[Xt@C̑召r
	 * @param o rl
	 * @return 0: 1: this > o -1: this < o
	 */
	public int compareTo(Object o) {
		DbFile dbf1 = this;
		DbFile dbf2 = (DbFile)o;

		// JnʒuŔr
		if(dbf1.getRecordBegin() == dbf2.getRecordBegin()) {
			return 0;
		}
		return dbf1.getRecordBegin() > dbf2.getRecordBegin() ? 1 : -1;
	}
}
