/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: restore.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.tools.console.command;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.logical_paradox.shell.Argument;
import org.logical_paradox.shell.Command;
import org.logical_paradox.shell.Option;
import org.logical_paradox.shell.ResultCode;
import org.logical_paradox.shell.ShellEnv;
import org.logical_paradox.shell.exception.CommandExecutionException;
import org.logical_paradox.shell.exception.TerminationException;
import org.logical_paradox.vsan.dac.Response;
import org.logical_paradox.vsan.dac.ResponseCode;
import org.logical_paradox.vsan.dac.command.Protocol;
import org.logical_paradox.vsan.tools.console.DacConsoleConstant;
import org.logical_paradox.vsan.tools.console.util.DumpListEditor;
import org.logical_paradox.vsan.tools.console.util.SocketWriter;

/**
 * <pre>
 * restore
 * 
 * ydlz
 * DACf[^M܂B
 * ̃R}h́AdacR}hɂĐݒ肳ꂽ'dac'ϐQƂ܂B
 * R[hԍ͔pJ}؂ŕ^邱Ƃł܂B
 * 
 * yz
 * restore --record=<record no.[,<record no.[..]]> [<hostname>[:<port>]]
 * 
 * </pre>
 * 
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class restore implements Command {
	public static final String USAGE = "restore --record=<record no.[,<record no.[..]]> [<hostname>[:<port>]]";

	public String usage() {
		return USAGE;
	}

	public ResultCode execute(OutputStream o, ShellEnv env, Argument a) throws CommandExecutionException, TerminationException {
		String dac = null;

		if(a == null || a.getOptions() == null || a.getOptions().length == 0) {
			throw new CommandExecutionException();
		}

		if(a == null || a.getArguments().length == 0) {
			dac = env.getProperty(DacConsoleConstant.ENV_DAC);
		} else if(a.getArguments().length > 1) {
			throw new CommandExecutionException();
		} else {
			dac = a.getArguments()[0];
		}

		// IvV̉
		try {
			Option[] options = a.getOptions();
			long[] records = null;

			for(int i = 0; i < options.length; i++) {
				Option option = options[i];
				if("record".equals(option.getName())) {
					records = parseRecords(option.getValue());
				} else {
					throw new CommandExecutionException();
				}
			}

			if(records == null || records.length == 0) {
				throw new CommandExecutionException();
			}

			/*
			 * Mf[^̍쐬
			 */
			ByteBuffer bb = ByteBuffer.allocate(1 + 4 + (records.length * 8)).order(ByteOrder.BIG_ENDIAN);
			bb.put((byte)Protocol.P_RESTORE);
			bb.putInt(records.length);

			// R[hXg
			for(int i = 0; i < records.length; i++) {
				bb.putLong(records[i]);
			}

			PrintWriter pw = new PrintWriter(o);
			pw.println("retrieving..\n");
			pw.flush();

			Response response = SocketWriter.send(dac, bb.array(), false);

			if(response.getResponseCode() == ResponseCode.OK) {
				// sXe[^XOK̏ꍇ
				displayDumpList(pw, response);
				pw.println("done.");
				pw.flush();
				pw.close();

				return new ResultCode(ResultCode.STATUS_OK);
			} else {
				// sXe[^XR[hOKȊȌꍇ
				pw.println("fatal error is caused in executing RESTORE command: code=[" + response.getResponseCode() + "]\n");
				pw.flush();
				pw.close();

				return new ResultCode(ResultCode.STATUS_ERROR);
			}
		} catch(IOException e) {
			throw new CommandExecutionException();
		}
	}
	/**
	 * DACMR[h̃f[^R\[ɏo͂
	 * @param pw o͐
	 * @param response DAC̉
	 */
	protected void displayDumpList(PrintWriter pw, Response response) {
		ByteBuffer buffer = ByteBuffer.allocate(response.getData().length).order(ByteOrder.BIG_ENDIAN);
		buffer.put(response.getData());
		buffer.rewind();

		// R[h
		int replyRecords = buffer.getInt();
		DumpListEditor dumpListEditor = new DumpListEditor();

		StringBuffer sb = new StringBuffer();
		for(int i = 0; i < replyRecords; i++) {
			// R[hǂ
			// ubNTCY / R[hԍ / f[^TCY
			int replyBlockSize = buffer.getInt();
			long recordNo = buffer.getLong();
			int datasize = buffer.getInt();
			byte[] dumpTarget = new byte[datasize];

			buffer.get(dumpTarget);

			sb.append("record no: " + recordNo + ", " + replyBlockSize + " bytes/block, ");

			sb.append(" size: " + datasize + " bytes\n");
			// _vo
			sb.append("-- -- -- -- -- -- -- --\n");
			sb.append(dumpListEditor.edit(dumpTarget));
			sb.append("\n");
			sb.append("-- -- -- -- -- -- -- --\n");
		}

		// R\[֏o͂
		pw.println(sb.toString());
	}
	/**
	 * J}؂̃R[hԍ͂
	 * @param line J}؂̐
	 * @return R[hԍ̔z
	 */
	protected long[] parseRecords(String line) {
		String[] numbers = line.split(",");
		long records[] = new long[numbers.length];

		for(int i = 0; i < records.length; i++) {
			records[i] = Long.parseLong(numbers[i]);
		}

		return records;
	}
}
