/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: status.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.tools.console.command;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.logical_paradox.shell.Argument;
import org.logical_paradox.shell.Command;
import org.logical_paradox.shell.ResultCode;
import org.logical_paradox.shell.ShellEnv;
import org.logical_paradox.shell.exception.CommandExecutionException;
import org.logical_paradox.shell.exception.TerminationException;
import org.logical_paradox.vsan.dac.Response;
import org.logical_paradox.vsan.dac.ResponseCode;
import org.logical_paradox.vsan.dac.command.Protocol;
import org.logical_paradox.vsan.tools.console.DacConsoleConstant;
import org.logical_paradox.vsan.tools.console.util.SocketWriter;

/**
 * <pre>
 * status
 * 
 * ydlz
 * DACɑ΂ĉғ|[gv܂B
 * ̃R}h́AdacR}hɂĐݒ肳ꂽ'dac'ϐQƂ܂B
 * 
 * yz
 * status [<hostname>[:<port>]]
 * 
 * </pre>
 * 
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class status implements Command {
	public static final String USAGE = "status [<hostname>[:<port>]]";

	public String usage() {
		return USAGE;
	}

	public ResultCode execute(OutputStream o, ShellEnv env, Argument a) throws CommandExecutionException, TerminationException {
		String dac = null;

		if(a == null || a.getArguments().length == 0) {
			dac = env.getProperty(DacConsoleConstant.ENV_DAC);
		} else if(a.getArguments().length > 1) {
			throw new CommandExecutionException();
		} else {
			dac = a.getArguments()[0];
		}
		if(a != null && a.getOptions().length > 0 || a != null && a.getArguments().length > 1) {
			throw new CommandExecutionException();
		}

		try {
			// Xe[^X|[gv
			PrintWriter pw = new PrintWriter(o);
			pw.println("retrieving controller status..\n");

			Response response = SocketWriter.send(dac, new byte[] {(byte)Protocol.P_STATUS_REPORT}, false);

			// Xe[^X|[g\
			if(response.getResponseCode() == ResponseCode.OK) {
				pw.println(new String(response.getData()) + "\n");

				pw.println("done.");
				pw.flush();
				pw.close();

				return new ResultCode(ResultCode.STATUS_OK);
			} else {
				// sXe[^XR[hOKȊȌꍇ
				pw.println("fatal error is caused in executing STATUS command: code=[" + response.getResponseCode() + "]\n");
				pw.flush();
				pw.close();

				return new ResultCode(ResultCode.STATUS_ERROR);
			}
		} catch(IOException e) {
			throw new CommandExecutionException(e.getMessage());
		}
	}

}
